/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */

/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#include "lib-lib.h"

#define BUFSIZ_INCREMENT  1024

void buffer_resize(buffer_t *buf, ssize_t newsize)
{
    if (newsize >= buf->size) {
        /* rounds newsize to the 1024 multiple just after newsize+1 */
        newsize = (newsize + BUFSIZ_INCREMENT) & ~(BUFSIZ_INCREMENT - 1);
        p_realloc(&buf->data, newsize);
    }
}




/****** LEGACY BUFFERS *******/

/*
 * Creates and initializes a BUFFER*. If passed an existing BUFFER*,
 * just initializes. Frees anything already in the buffer.
 *
 * Disregards the 'destroy' flag, which seems reserved for caller.
 * This is bad, but there's no apparent protocol for it.
 */
BUFFER *mutt_buffer_init(BUFFER *b)
{
    if (!b) {
        b = p_new(BUFFER, 1);
    }
    p_delete(&b->data);
    p_clear(b, 1);
    return b;
}

/*
 * Creates and initializes a BUFFER*. If passed an existing BUFFER*,
 * just initializes. Frees anything already in the buffer. Copies in
 * the seed string.
 *
 * Disregards the 'destroy' flag, which seems reserved for caller.
 * This is bad, but there's no apparent protocol for it.
 */
BUFFER *mutt_buffer_from(BUFFER * b, const char *seed)
{
    if (!seed)
        return NULL;

    b = mutt_buffer_init(b);
    b->dsize = m_strlen(seed);
    b->data  = m_strdup(seed);
    b->dptr  = (char *)b->data + b->dsize;
    return b;
}

void mutt_buffer_free(BUFFER **p)
{
    if (p && *p) {
        p_delete(&(*p)->data);
        p_delete(p);
    }
}

/* dynamically grows a BUFFER to accomodate s, in increments of 128 bytes.
 * Always one byte bigger than necessary for the null terminator, and
 * the buffer is always null-terminated */
void mutt_buffer_add(BUFFER *buf, const char *s, ssize_t len)
{
    size_t offset;

    if (buf->dptr + len + 1 > buf->data + buf->dsize) {
        offset = buf->dptr - buf->data;
        buf->dsize += ((len + 1 + 127) & ~127);
        p_realloc(&buf->data, buf->dsize);
        buf->dptr = buf->data + offset;
    }
    memcpy(buf->dptr, s, len);
    buf->dptr += len;
    *buf->dptr = '\0';
}

